/*
 *  FILE: File handling routines for LLC
 * 
 *  NOTE ABOUT FILE HANDLING IN LLC: A file is a FILE *. For the ROOT processor,
 *  a FILE * is effectively opened as usual in C (NULL if none, != 0 if
 *  succesfull). For the rest of the processors, a FILE * is also NULL if it
 *  has not been succesfully opened, but != 0 otherwise. However, THIS NUMBER
 *  DOESN'T MEAN a file handler, since only the ROOT processor can handle files.
 *  Don't worry about that; all the file handling is proccesed by LLC functions
 *  transparently, but take care of only using FILE ptr's with LLC funcs.
 *
 *  LAST CHANGES:
 *      (18-aug-98) File handling routines fixed for working with MPI
 *      (21-aug-98) Fixed problems with  # in Dig ARCH
 *      (7-sept-98) GF... broadcasting of results improved
 */

#ifndef LLCFILESINCLUDED
#define LLCFILESINCLUDED

#include "llcsync.h"

#ifdef CCSEQ
#   define GFOPEN fopen
#   define GFCLOSE fclose
#   define GFEOF  feof
#   define GFERROR ferror
#   define GFFLUSH fflush
#   define GFGETC fgetc
#   define GFGETS fgets
#   define FPUTC fputc
#   define FPUTS fputs
#   define FSEEK fseek
#   define FWRITE fwrite
#   define GFREAD fread
#else

/*  --- FILE open ---
 *  RECEIVES: Same as the fopen() call
 *  RETURNS: Same as fopen() call
 *  G function
 */
FILE *GFOPEN (char *path, char *mode)
{
  FILE *fileptr, aux;
  int i, envio;

  if (LLNAME == 0) {
    fileptr = fopen(path, mode);
    aux = aux;
    envio = (fileptr == NULL)? 0 : 1;
  }
  else {
    path = path;
    mode = mode;
  }

  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SENDINT(i, envio);
      else
        RECEIVEINT(i, envio);
  if (LLNAME != 0)
    fileptr = (envio == 0)? NULL : &aux;
  return (fileptr);
}

/*  --- FILE close ---
 *  RECEIVES AND RETURNS: Same as fclose
 *  G Function
 */
int GFCLOSE (FILE *name)
{
  int i, value;

  if (LLNAME == 0) 
    value = fclose(name);
  else {
    value = 0;
    name = name;
  }
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SENDINT(i, value);
      else
        RECEIVEINT(i, value);
  return (value);
}

/*  --- FILE EOF reached ---
 *  RECEIVES: File * to ask about
 *  RETURNS: Same as feof() call
 *  G Function: Every processor receives the same value.
 */
int GFEOF (FILE *name)
{
  int i, value;

  value = 0;
  if (LLNAME == 0)
    value = feof(name);
  else
    name = name;
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SENDINT(i, value);
      else
        RECEIVEINT(i, value);
  return (value);
}

/*  --- FILE Error status ---
 *  RECEIVES: File * to ask about
 *  RETURNS: Same as ferror() call
 *  G Function: Every processor receives the same value.
 */
int GFERROR (FILE *name)
{
  int i, value;

  if (LLNAME == 0)
    value = ferror(name);
  else {
    value = 0;
    name = name;
  }
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SENDINT(i, value);
      else
        RECEIVEINT(i, value);
  return (value);
}

/*  --- FILE Flush ---
 *  RECEIVES: File * to flush buffer
 *  RETURNS: Same as fflush() call
 *  G Function
 */
int GFFLUSH (FILE *name)
{
  int i, value;

  if (LLNAME == 0)
    value = fflush(name);
  else {
    value = 0;
    name = name;
  }
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SENDINT(i, value);
      else
        RECEIVEINT(i, value);
  return (value);
}

/*  --- Gets a char from FILE ---
 *  RECEIVES: File *
 *  RETURNS: Same as fgetc() call
 *  G Function: Every processor receives the char.
 */
char GFGETC (FILE *name)
{
  int i;
  char value;

  if (LLNAME == 0)
    value = fgetc(name);
  else
    name = name;
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SEND(i, &value, sizeof(char));
      else
        RECEIVE(i, &value, sizeof(char));
  return (value);
}

/*  --- Gets a string from FILE ---
 *  RECEIVES: Same as fgets().
 *  RETURNS: Same as fgets() call
 *  G Function: Every processor receives the same string.
 */
char *GFGETS (char *s, int n, FILE *name)
{
  int i;

  if (LLNAME == 0)
    s = fgets(s, n, name);
  else {
    s = s;
    n = n;
    name = name;
  }

  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0)
        SEND(i, s,n);
      else
        RECEIVE(i, s, n);
  return (s);
}

/*  --- Puts a char into FILE ---
 *  RECEIVES: Same as fputc().
 *  RETURNS: Same as fputc() call (Only works on ROOT processor)
 */
char FPUTC (int c, FILE *name)
{
  int value = 0;

  if (LLNAME == 0)
    value = fputc(c, name);
  else {
    c = c;
    name = name;
  }
  return (value);
}

/*  --- Puts a string into FILE ---
 *  RECEIVES: Same as fputs().
 *  RETURNS: Same as fputs() call (Only works on ROOT processor)
 */
int FPUTS (char *s, FILE *name)
{
  int value = 0;

  if (LLNAME == 0)
    value = fputs(s, name);
  else {
    s = s;
    name = name;
  }
  return (value);
}

/*  --- Reads a block of bytes from FILE ---
 *  RECEIVES: Same as fread().
 *  RETURNS: Same as fread() call
 *  G Function: Every processor receives the same block
 */
int GFREAD (char *s, int size, int nmemb, FILE *name)
{
  int i;
  int value;

  if (LLNAME == 0)
    value = fread(s, size, nmemb, name);
  else {
    s = s;
    size = size;
    nmemb = nmemb;
    name = name;
  }
  for (i = 0; i < bit; i++)
    if ((LLNAME < (1 << (i+1))))
      if ((LLNAME & (1 << i)) == 0) {
	     SENDINT(i, size);
	     SENDINT(i, nmemb);
	     SEND(i, s, size*nmemb);
      }
      else
      {
	     RECEIVEINT(i, size);
	     RECEIVEINT(i, nmemb);
	     RECEIVE(i, s, size*nmemb);
      }
  return (value);
}

/*  --- Writes a block of bytes into FILE ---
 *  RECEIVES: Same as fwrite().
 *  RETURNS: Same as fwrite() call; Only works on ROOT
 */
int FWRITE (char *s, int size, int nmemb, FILE *name)
{
  int value = 0;

  if (LLNAME == 0)
    value = fwrite(s, size, nmemb, name);
  else {
    s = s;
    size = size;
    nmemb = nmemb;
    name = name;
  }
  return (value);
}



/*  --- fseek a filepos in FILE ---
 *  RECEIVES: Same as fseek().
 *  RETURNS: Same as fseek() call. Only works on ROOT
 */
int FSEEK (FILE *name, long int offset, int whence)
{
  int value = 0;

  if (LLNAME == 0)
    value = fseek(name, offset, whence);
  else {
    offset = offset;
    whence = whence;
    name = name;
  }
  return (value);
}

#endif /* ifndef CCSEQ */

#endif /* ifndef PPCFILESINCLUDED */


