/*
 * FILE: LLC PRINTF and GPRINTF handling functions
 * LAST MODIFIED: 21-8-98
 * LAST CHANGES:
 *      (9-aug-98): MPI PRINTF and GPRINTF added
 *     (21-aug-98): fixed problems with # in dig ARCH
 */

#ifndef LLCPRINTFINCLUDED
#define LLCPRINTFINCLUDED

#include "llcsync.h"

#ifndef CCSEQ /* Nothing of this needs to exist in a sequential model */

#if defined(CCPVM) || defined(CCTRANSPUTER)

/******* Level 2 Functions: GPRINTF related ******/
typedef struct llstrings
{
	char *msg;
	struct llstrings *next;
} LLSTRING;  /* - Node of the queue of GPRINTF messages - */

LLSTRING *LLBuffHead,        /* Head and */
			*LLBuffTail = NULL; /* Tail of the queue of GPRINTF messages */
char *llstringglobal;


#ifdef CCTRANSPUTER
	/* --- Creates the communication channels at the beginning (called from INITIALIZE) ---
	 * RECEIVES: The old comm channels, and the new number of channels to allocate
	 * RETURN: The new (m)allocated channels
	 */
	Channel **CreateChannels (Channel *OldChannels [], int ChannelsSize)
	{
		Channel **NewChannels = NULL;

		if ((NewChannels = malloc ((ChannelsSize + 1) * sizeof (Channel *))) == NULL)
			abort ();
		else
			{
			int ChannelsCount = 0;

			while (ChannelsCount++ < ChannelsSize)
				NewChannels [ChannelsCount - 1] = OldChannels [ChannelsCount - 1];
			NewChannels[ChannelsSize] = NULL;
			}
		return (NewChannels);
	}
#endif

int node_level (int name)
{
  register int lev = 0, max_name = 0;

  while (max_name < name)
  {
	 lev++;
	 max_name = (max_name << 1) + 1;
  }
  return (lev);
}

/* --- Printf for the root process ---
 * RECEIVES: Same as the standard printf func
 * RETURNS: If ROOT, prints the args as usual.
 * 			Else, just skips
 */
void PRINTF (char *format, ...)
{
	char aux_string[SIZEBUFFER];

	va_list vargs;
	va_start(vargs, format);
	aux_string [vsprintf(aux_string, format, vargs)] = '\0';
	va_end(vargs);
#       ifdef ROOT
	   printf (aux_string);
#       endif
}

/* --- Printf for a particular process ---
 * RECEIVES: Same as the standard printf func, plus the number of the
 * 			 processor that prints the message.
 */
void PPRINTF (int procid, char *format, ...)
{
	char aux_string[SIZEBUFFER];
	int len;

	va_list vargs;
	va_start(vargs, format);

	if (LLNAME == procid) {
		aux_string [len = vsprintf(aux_string, format, vargs)] = '\0';
		len++;
		}

	if (procid != 0)
		GSEND(procid, 0, aux_string, len);

#       ifdef ROOT
		printf (aux_string);
#       endif

	va_end(vargs);
}

#if defined(CCTRANSPUTER)
/* --- Definition of strdup - InmosC for DOS lacks of it --- */
char *strdup(const char *s) {
	 char *temp;

	 temp = malloc(strlen(s) + 1);
	 if (temp != NULL)
		 strcpy(temp, s);
	 return (temp);
}
#endif

/* --- Adds a printf call to the output queue ---
 * RECEIVES: The params for the printf call
 * RETURNS: Updates the queue
 */
void llinsert_string (char *aux)
{
	LLSTRING *newstring;

	if ((newstring = (LLSTRING *) malloc (sizeof (LLSTRING))) == NULL)
		abort();
	newstring->next = NULL;
	newstring->msg = (char *)strdup(aux);
	if (newstring->msg == NULL)
		abort();

	if (LLBuffHead == NULL)
		LLBuffHead = LLBuffTail = newstring;
	else
	{
	  if (LLBuffHead == LLBuffTail)
	  {
		 LLBuffHead->next = newstring;
		 LLBuffTail = newstring;
	  }
	  else
	  {
		 LLBuffTail->next = newstring;
		 LLBuffTail = newstring;
	  }
	}
}


/* --- Frees the buffer of messages for the PRINTF func ---
 * RECEIVES: Nothing
 * RETURNS: Both LLBufTail and LLBufHead points to NULL
 */
void llfree_LLBuff (void)
{
	LLSTRING *dead;

	while (LLBuffHead != NULL)
	{
		dead = LLBuffHead;
		LLBuffHead = LLBuffHead->next;
		free (dead->msg);
		free (dead);
	}
	LLBuffTail = NULL;
}


/* --- Concatenates the whole PRINTF output buffer in a single string ---
 * RECEIVES: The output string, as calculated by llstring_len
 * RETURNS: In llptr, the whole output buffer.
 * 			Frees the output buffer (calls to llfree_LLBuffer)
 */
void llconcat_string(char *llptr)
{
	LLSTRING *present;

	llptr[0] = '\0';
	present = LLBuffHead;
	while (present != NULL)
	{
		strcat (llptr, present->msg);
		present = present->next;
	}
	llfree_LLBuff();
}

/* --- Calculates the buffer length needed for the llconcat_string call ---
 * RECEIVES: Nothing
 * RETURNS: The number of bytes needed for a buffer that would contain the
 * 			contents of the whole output buffer
 */
int llstring_len (void)
{
	LLSTRING *present;
	int len = 0;

	present = LLBuffHead;
	while (present != NULL)
	{
		len += strlen (present->msg);
		present = present->next;
	}
  return (len);
}

/* --- Global Printf ---
 * RECEIVES: Same params as the printf call
 * RETURNS: Nothing
 */
void GPRINTF(char *format, ...)
{
	char aux_string[SIZEBUFFER];
	int len, x, i;
	va_list vargs;

	va_start(vargs, format);
	len = vsprintf(aux_string, format, vargs);
	aux_string[len] = '\0';

	if (len != 0)
	  llinsert_string (aux_string);
	va_end(vargs);

	for (i = 0; i < bit; i++)
	{
		if (LLNAME & (1 << i))
		{
			len = llstring_len();
			if ((llstringglobal = (char *) malloc ((len + 1)*sizeof(char))) == NULL)  abort();
			llconcat_string(llstringglobal);
			llstringglobal[len] = '\0';       /* Incluir cero final */
				SENDINT(i, len);
			if (len != 0)
				SEND(i, llstringglobal, len);
			free (llstringglobal);
			break;
		}
		else
		{
			RECEIVEINT(i, x);
			if (x != 0)
			{
				RECEIVE(i, aux_string, x);
				aux_string[x] = '\0';
				llinsert_string (aux_string);
			}
		}
#               ifdef ROOT
		len = llstring_len();
		if ((llstringglobal = (char *) malloc ((len + 1)*sizeof(char))) == NULL)  abort();
			llconcat_string(llstringglobal);
		llstringglobal[len] = '\0';       /* Incluir cero final */
			PRINTF("%s", llstringglobal);
		free (llstringglobal);
#               endif
	}
}

#else /* MPI functions */

/* Not much pretty, but works */
#define GPRINTF printf

/* --- PRINTF --- */
void PRINTF(char *format, ...)
{
char buffer[1000];
int len;
va_list vargs;
  va_start(vargs, format);
  buffer[vsprintf(buffer, format, vargs)] = '\0';
  va_end(vargs);
  if (NAME == 0)
     printf(buffer);
}

#endif /* #if defined(CCPVM) || defined(CCTRANSPUTER) */

/* --- Boolean PRINTF ---
 * RECEIVES: A boolean condition B, and the isame params as the PRINTF func.
 * RETURNS: If B, prints the parameters.
 *          else, skips.
 */
void BPRINTF(int B, char *format, ...)
{
char buffer[SIZEBUFFER];
int len;
va_list vargs;

	va_start(vargs, format);
	buffer[vsprintf(buffer, format, vargs)] = '\0';
	va_end(vargs);

   if (B)
      GPRINTF(buffer);
   else
      GPRINTF("");
}


#endif /* #ifndef CCSEQ */

#endif  /* #ifndef LLCPRINTFINCLUDED */
